<?php
/**
 * Executes functions in the Football Pool plugin.
 */

WP_CLI::add_command( 'football-pool', 'Football_Pool_CLI_Command' );

class Football_Pool_CLI_Command extends WP_CLI_Command {
	/**
	 * Calculates the user ranking.
	 *
	 * ## OPTIONS
	 *
	 * [--force-calculation]
	 * : To force a calculation (see help page in plugin for extra info).
	 *
	 * ## EXAMPLES
	 *
	 *     wp football-pool calc
	 *
	 *     wp football-pool calc --force-calculation
	 *
	 * @alias calculate
	 */
	public function calc( $args, $assoc_args ) {
		$force_calculation = ( isset( $assoc_args['force-calculation'] ) && $assoc_args['force-calculation'] === true ) ? 1 : 0;
		$calc_args = array(
			'force_calculation' => $force_calculation,
			'iteration' => 0,
		);

		$calc_args = Football_Pool_Admin_Score_Calculation::process( true, $calc_args );
		extract( $calc_args, EXTR_OVERWRITE );
		
		$progress = \WP_CLI\Utils\make_progress_bar( 'Calculating scores', $total_iterations );
		// already tick one because we did the prepare step
		$progress->tick();
		// loop through rest of steps
		while ( $completed !== 1 && $error === false ) {
			$calc_args = Football_Pool_Admin_Score_Calculation::process( true, $calc_args );
			extract( $calc_args, EXTR_OVERWRITE );
			$progress->tick();
		}
		// on finish
		if ( $completed === 1 ) $progress->finish();
		
		if ( $error !== false ) {
			WP_CLI::error( $message );
		} else {
			if ( isset( $message_type ) && $message_type == 'warning' ) {
				WP_CLI::warning( $message );
			} else {
				WP_CLI::success( $message );
			}
		}
	}
}
