<?php
require_once( '../../../../../../wp-load.php' );
require_once( '../../../define.php' );

$site_url = get_option( 'siteurl' );
$admin_url = get_admin_url();
$tinymce_url = $site_url . '/wp-includes/js/tinymce/';
$tinymce_ver = get_bloginfo( 'version' );
$pool = new Football_Pool_Pool();

function group_options() {
	$groups = Football_Pool_Groups::get_groups();
	foreach ( $groups as $group ) {
		printf( '<option value="%d">%s</option>', $group->id, $group->name );
	}
}

function ranking_options() {
	global $pool;
	$rankings = $pool->get_rankings( 'user defined' );
	foreach ( $rankings as $ranking ) {
		printf( '<option value="%d">%s</option>', $ranking['id'], $ranking['name'] );
	}
}

function bonusquestion_options() {
	global $pool;
	$questions = $pool->get_bonus_questions();
	foreach( $questions as $question ) {
		if ( $question['match_id'] == 0 ) {
			printf( '<option value="%d">%d: %s</option>', $question['id'], $question['id'], $question['question'] );
		}
	}
}

function match_options() {
	$matches = new Football_Pool_Matches();
	$matches = $matches->matches;
	foreach ( $matches as $match ) {
		$option_text = sprintf( '%d: %s - %s (%s)'
								, $match['id']
								, $match['home_team']
								, $match['away_team']
								, Football_Pool_Utils::date_from_gmt( $match['date'] )
						);
		printf( '<option value="%d">%s</option>', $match['id'], $option_text );
	}
}
?>
<html xmlns="http://www.w3.org/1999/xhtml">
<head>
	<title><?php _e( 'Select a shortcode', 'football-pool' ); ?></title>
	<meta http-equiv="Content-Type" content="<?php bloginfo( 'html_type' ); ?>; charset=<?php echo get_option( 'blog_charset' ); ?>" />
	<script src="//ajax.googleapis.com/ajax/libs/jquery/1.9.1/jquery.min.js"></script>
	<script src="<?php echo $tinymce_url; ?>tiny_mce_popup.js?ver=<?php echo $tinymce_ver; ?>"></script>
	<script src="<?php echo $tinymce_url; ?>utils/mctabs.js?ver=<?php echo $tinymce_ver; ?>"></script>
	<script src="<?php echo FOOTBALLPOOL_PLUGIN_URL ?>assets/admin/admin.min.js?ver=<?php echo FOOTBALLPOOL_DB_VERSION; ?>"></script>
	<base target="_self" />
	
	<style type="text/css">
	#tabs li:focus { outline-width: 0; }
	#tabs li, .dialog-table label { cursor: pointer; }
	.dialog-table { border: 0; border-collapse: collapse; }
	.dialog-table td { padding: 3px 5px 3px 3px; vertical-align: top; }
	/* .dialog-table td * { font-size: 12px!important; } */
	.dialog-table td p { margin-bottom: 3px; }
	#panel_wrapper div.current { height: 270px; }
	.atts { display: none; }
	.shortcode-select { background-color: #f1f1f1; }
	.shortcode-select td p { margin-bottom: 8px; }
	.shortcode-select td { font-weight: bold; }
	.info { font-style: italic; }
	.tr-count input[type=text] { width: 150px; }
	</style>
	
	<script>
	jQuery( document ).ready( function() {
		tinyMCEPopup.executeOnLoad( 'FootballPoolAdmin.tinymce_init()' );
		FootballPoolAdmin.tinymce_init_tabs( 'tabs' );
	});
	
	function toggle_atts( select_id, atts ) {
		var selected_val = jQuery( '#' + select_id ).val();
		jQuery.each( atts, function( key, value ) {
			if ( selected_val == key ) {
				jQuery( '.' + value ).show();
			} else {
				jQuery( '.' + value ).hide();
			}
		});
	}
	
	function toggle_count_texts( id ) {
		var text_ids = ['#text-1', '#text-2', '#text-3', '#text-4'];
		if ( jQuery( '#' + id ).is( ':checked' ) ) {
			FootballPoolAdmin.set_input_param( 'placeholder', text_ids, 'none' );
		} else {
			FootballPoolAdmin.restore_input_param( 'placeholder', text_ids );
		}
		FootballPoolAdmin.disable_inputs( text_ids, id );
	}
	
	function toggle_select_row( clicked, shortcode ) {
		clicked = jQuery( clicked ).attr( 'for' );
		jQuery( '.tr-' + shortcode + ' select' ).each( function() {
			if ( jQuery( this ).attr( 'id' ) == clicked ) {
				jQuery( this ).show( 'slow' );
			} else {
				jQuery( this ).hide( 'slow' );
			}
		});
	}
	</script>
</head>
<body>
<form>
	<div id="tabs" class="tabs">
		<ul>
			<li id="pool_tab" class="current"><span><?php _e( 'Pool', 'football-pool' ); ?></span></li>
			<li id="links_tab"><span><?php _e( 'Links', 'football-pool' ); ?></span></li>
			<li id="other_tab"><span><?php _e( 'Other', 'football-pool' ); ?></span></li>
		</ul>
	</div>

	<div id="panel_wrapper" class="panel_wrapper">
		<!-- panel -->
		<div id="pool_panel" class="panel current"><br/>
			<table class="dialog-table">
			<tr class="shortcode-select">
				<td><label for="s-pool"><?php _e( 'Select a shortcode', 'football-pool' ); ?></label></td>
				<td>
					<select id="s-pool" class="shortcode" onchange="toggle_atts( this.id, { 'fp-ranking': 'tr-ranking', 'fp-group': 'tr-group', 'fp-predictionform': 'tr-predictionform', 'fp-user-score': 'tr-user-score', 'fp-user-ranking': 'tr-user-ranking', 'fp-predictions': 'tr-predictions', 'fp-matches': 'tr-matches', 'fp-next-matches': 'tr-next-matches', 'fp-league-info': 'tr-league-info' } )">
						<option value="fp-ranking"><?php _e( 'Ranking', 'football-pool' ); ?></option>
						<option value="fp-predictions"><?php _e( 'Predictions for match or question', 'football-pool' ); ?></option>
						<option value="fp-user-score"><?php _e( 'Score for a user', 'football-pool' ); ?></option>
						<option value="fp-user-ranking"><?php _e( 'Ranking for a user', 'football-pool' ); ?></option>
						<option value="fp-group"><?php _e( 'Group', 'football-pool' ); ?></option>
						<option value="fp-predictionform"><?php _e( 'Prediction form', 'football-pool' ); ?></option>
						<option value="fp-matches"><?php _e( 'Matches', 'football-pool' ); ?></option>
						<option value="fp-next-matches"><?php _e( 'Next matches', 'football-pool' ); ?></option>
						<option value="fp-league-info"><?php _e( 'League info', 'football-pool' ); ?></option>
					</select>
				</td>
			</tr>
			<!-- fp-ranking -->
			<tr class="tr-ranking">
				<td>
					<label for="ranking-id"><?php _e( 'Select a ranking', 'football-pool' ); ?></label>
				</td>
				<td>
					<select id="ranking-id">
						<optgroup label="<?php _e( 'default', 'football-pool' ); ?>">
							<option value="0" selected="selected"><?php _e( 'all scores', 'football-pool' ); ?></option>
						</optgroup>
						<optgroup label="<?php _e( 'or choose a user defined ranking', 'football-pool' ); ?>">
							<?php ranking_options(); ?>
						</optgroup>
					</select>
				</td>
			</tr>
			<tr class="tr-ranking">
				<td>
					<label for="ranking-league"><?php _e( 'Select a league', 'football-pool' ); ?></label>
				</td>
				<td>
					<select id="ranking-league">
						<optgroup label="<?php _e( 'default', 'football-pool' ); ?>">
							<option value="0" selected="selected"><?php _e( 'all players', 'football-pool' ); ?></option>
						</optgroup>
						<optgroup label="<?php _e( 'or choose a league', 'football-pool' ); ?>">
							<?php
							$leagues = $pool->get_leagues( true );
							foreach ( $leagues as $league ) {
								printf( '<option value="%d">%s</option>'
										, $league['league_id'], $league['league_name'] 
								);
							}
							?>
						</optgroup>
					</select>
				</td>
			</tr>
			<tr class="tr-ranking">
				<td>
					<label for="ranking-num"><?php _e( 'Number of players', 'football-pool' ); ?></label>
				</td>
				<td>
					<input type="text" id="ranking-num" placeholder="5" />
				</td>
			</tr>
			<tr class="tr-ranking">
				<td>
					<label for="ranking-show-num-predictions"><?php _e( 'Show number of predictions?', 'football-pool' ); ?></label>
				</td>
			</tr>
			<tr class="tr-ranking">
				<td>
					<label><a href="//php.net/manual/en/function.date.php" title="<?php _e( 'information about PHP\'s date format', 'football-pool' ); ?>" target="_blank"><?php _e( 'Date', 'football-pool' ); ?></a></label>
				</td>
				<td>
					<label><input type="radio" id="ranking-date-now" name="ranking-date" value="now" checked="checked" /> <?php _e( 'now', 'football-pool' ); ?></label><br />
					<label><input type="radio" id="ranking-date-postdate" name="ranking-date" value="postdate" /> <?php _e( 'postdate', 'football-pool' ); ?></label><br />
					<label><input type="radio" id="ranking-date-custom" name="ranking-date" value="custom" onclick="jQuery( '#ranking-date-custom-value' ).focus();" /> <?php _e( 'custom date', 'football-pool' ); ?>:</label> <input type="text" id="ranking-date-custom-value" placeholder="Y-m-d H:i" onclick="jQuery( '#ranking-date-custom' ).prop( 'checked', true );" /><br />
				</td>
			</tr>
			<!-- fp-predictions -->
			<tr class="tr-predictions atts">
				<td>
					<label for="predictions-match"><?php _e( 'Match', 'football-pool' ); ?></label>
				</td>
				<td>
					<select id="predictions-match">
						<option value="0"><?php _e( 'Select a match', 'football-pool' ); ?></option>
						<?php match_options(); ?>
					</select>
				</td>
			</tr>
			<tr class="tr-predictions atts">
				<td></td><td class="info"><?php _e( 'and/or', 'football-pool' ); ?></td>
			</tr>
			<tr class="tr-predictions atts">
				<td>
					<label for="predictions-question"><?php _e( 'Question', 'football-pool' ); ?></label>
				</td>
				<td>
					<select id="predictions-question">
						<option value="0"><?php _e( 'Select a question', 'football-pool' ); ?></option>
						<?php bonusquestion_options(); ?>
					</select>
				</td>
			</tr>
			<tr class="tr-predictions atts">
				<td>
					<label for="predictions-text"><?php _e( 'Text', 'football-pool' ); ?></label>
				</td>
				<td>
					<input type="text" id="predictions-text" style="width:100%" placeholder="<?php _e( 'Text to display if there is nothing to show', 'football-pool' ); ?>" />
				</td>
			</tr>
			<!-- fp-user-score -->
			<tr class="tr-user-score atts">
				<td>
					<label for="user-score-user-id"><?php _e( 'Select a user', 'football-pool' ); ?></label>
				</td>
				<td>
					<select id="user-score-user-id">
						<optgroup label="<?php _e( 'default', 'football-pool' ); ?>">
							<option value="" selected="selected"><?php _e( 'logged in user', 'football-pool' ); ?></option>
						</optgroup>
						<optgroup label="<?php _e( 'or choose another user', 'football-pool' ); ?>">
							<?php
							$users = $pool->get_users( 1 );
							foreach ( $users as $user ) {
								printf( '<option value="%d">%s</option>', $user['user_id'], $user['user_name'] );
							}
							?>
						</optgroup>
					</select>
				</td>
			</tr>
			<tr class="tr-user-score atts">
				<td>
					<label for="user-score-ranking-id"><?php _e( 'Select a ranking', 'football-pool' ); ?></label>
				</td>
				<td>
					<select id="user-score-ranking-id">
						<optgroup label="<?php _e( 'default', 'football-pool' ); ?>">
							<option value="0" selected="selected"><?php _e( 'all scores', 'football-pool' ); ?></option>
						</optgroup>
						<optgroup label="<?php _e( 'or choose a user defined ranking', 'football-pool' ); ?>">
							<?php ranking_options(); ?>
						</optgroup>
					</select>
				</td>
			</tr>
			<tr class="tr-user-score atts">
				<td>
					<label for="user-score-text"><?php _e( 'Text', 'football-pool' ); ?></label>
				</td>
				<td>
					<input type="text" id="user-score-text" placeholder="0" />
				</td>
			</tr>
			<tr class="tr-user-score atts">
				<td>
					<label><a href="//php.net/manual/en/function.date.php" title="<?php _e( 'information about PHP\'s date format', 'football-pool' ); ?>" target="_blank"><?php _e( 'Date', 'football-pool' ); ?></a></label>
				</td>
				<td>
					<label><input type="radio" id="user-score-date-now" name="user-score-date" value="now" checked="checked" /> <?php _e( 'now', 'football-pool' ); ?></label><br />
					<label><input type="radio" id="user-score-date-postdate" name="user-score-date" value="postdate" /> <?php _e( 'postdate', 'football-pool' ); ?></label><br />
					<label><input type="radio" id="user-score-date-custom" name="user-score-date" value="custom" onclick="jQuery( '#user-score-date-custom-value' ).focus();" /> <?php _e( 'custom date', 'football-pool' ); ?>:</label> <input type="text" id="user-score-date-custom-value" placeholder="Y-m-d H:i" onclick="jQuery( '#user-score-date-custom' ).prop( 'checked', true );" /><br />
				</td>
			</tr>
			<!-- fp-user-ranking -->
			<tr class="tr-user-ranking atts">
				<td>
					<label for="user-ranking-user-id"><?php _e( 'Select a user', 'football-pool' ); ?></label>
				</td>
				<td>
					<select id="user-ranking-user-id">
						<optgroup label="<?php _e( 'default', 'football-pool' ); ?>">
							<option value="" selected="selected"><?php _e( 'logged in user', 'football-pool' ); ?></option>
						</optgroup>
						<optgroup label="<?php _e( 'or choose another user', 'football-pool' ); ?>">
							<?php
							$users = $pool->get_users( 1 );
							foreach ( $users as $user ) {
								printf( '<option value="%d">%s</option>', $user['user_id'], $user['user_name'] );
							}
							?>
						</optgroup>
					</select>
				</td>
			</tr>
			<tr class="tr-user-ranking atts">
				<td>
					<label for="user-ranking-ranking-id"><?php _e( 'Select a ranking', 'football-pool' ); ?></label>
				</td>
				<td>
					<select id="user-ranking-ranking-id">
						<optgroup label="<?php _e( 'default', 'football-pool' ); ?>">
							<option value="0" selected="selected"><?php _e( 'all scores', 'football-pool' ); ?></option>
						</optgroup>
						<optgroup label="<?php _e( 'or choose a user defined ranking', 'football-pool' ); ?>">
							<?php ranking_options(); ?>
						</optgroup>
					</select>
				</td>
			</tr>
			<tr class="tr-user-ranking atts">
				<td>
					<label for="user-ranking-text"><?php _e( 'Text', 'football-pool' ); ?></label>
				</td>
				<td>
					<input type="text" id="user-ranking-text" placeholder="" />
				</td>
			</tr>
			<tr class="tr-user-ranking atts">
				<td>
					<label><a href="//php.net/manual/en/function.date.php" title="<?php _e( 'information about PHP\'s date format', 'football-pool' ); ?>" target="_blank"><?php _e( 'Date', 'football-pool' ); ?></a></label>
				</td>
				<td>
					<label><input type="radio" id="user-ranking-date-now" name="user-ranking-date" value="now" checked="checked" /> <?php _e( 'now', 'football-pool' ); ?></label><br />
					<label><input type="radio" id="user-ranking-date-postdate" name="user-ranking-date" value="postdate" /> <?php _e( 'postdate', 'football-pool' ); ?></label><br />
					<label><input type="radio" id="user-ranking-date-custom" name="user-ranking-date" value="custom" onclick="jQuery( '#user-ranking-date-custom-value' ).focus();" /> <?php _e( 'custom date', 'football-pool' ); ?>:</label> <input type="text" id="user-ranking-date-custom-value" placeholder="Y-m-d H:i" onclick="jQuery( '#user-ranking-date-custom' ).prop( 'checked', true );" /><br />
				</td>
			</tr>
			<!-- fp-group -->
			<tr class="tr-group atts">
				<td>
					<label for="group-id"><?php _e( 'Select a group', 'football-pool' ); ?></label>
				</td>
				<td>
					<select id="group-id">
					<?php group_options(); ?>
					</select>
				</td>
			</tr>
			<!-- fp-predictionform -->
			<tr class="tr-predictionform atts">
				<td colspan="2">
					<strong><?php _e( 'Click a label to show the options.', 'football-pool' );?></strong>
					<br />
					<?php _e( 'Use CTRL+click to select multiple values.', 'football-pool' );?>
				</td>
			<tr>
			<tr class="tr-predictionform atts">
				<td>
					<label for="match-id" onclick="toggle_select_row( this, 'predictionform' )"><?php _e( 'Select one or more matches', 'football-pool' ); ?></label>
				</td>
				<td>
					<select id="match-id" style="width:320px; height:100px; display:none;" multiple="multiple">
					<?php match_options(); ?>
					</select>
				</td>
			</tr>
			<tr class="tr-predictionform atts">
				<td>
					<label for="matchtype-id" onclick="toggle_select_row( this, 'predictionform' )"><?php _e( 'Select one or more match types', 'football-pool' ); ?></label>
				</td>
				<td>
					<select id="matchtype-id" style="width:320px; height:100px; display:none;" multiple="multiple">
					<?php
					$match_types = Football_Pool_Matches::get_match_types();
					foreach( $match_types as $match_type ) {
						printf( '<option value="%d">%s</option>', $match_type->id, $match_type->name );
					}
					?>
					</select>
				</td>
			</tr>
			<tr class="tr-predictionform atts">
				<td>
					<label for="question-id" onclick="toggle_select_row( this, 'predictionform' )"><?php _e( 'Select one or more questions', 'football-pool' ); ?></label>
				</td>
				<td>
					<select id="question-id" style="width:320px; height:100px; display:none;" multiple="multiple">
					<?php bonusquestion_options(); ?>
					</select>
				</td>
			</tr>
			<!-- fp-matches -->
			<tr class="tr-matches atts">
				<td colspan="2">
					<strong><?php _e( 'Click a label to show the options.', 'football-pool' );?></strong>
					<br />
					<?php _e( 'Use CTRL+click to select multiple values.', 'football-pool' );?>
				</td>
			<tr>
			<tr class="tr-matches atts">
				<td>
					<label for="matches-match-id" onclick="toggle_select_row( this, 'matches' )"><?php _e( 'Select one or more matches', 'football-pool' ); ?></label>
				</td>
				<td>
					<select id="matches-match-id" style="width:320px; height:100px; display:none;" multiple="multiple">
					<?php match_options(); ?>
					</select>
				</td>
			</tr>
			<tr class="tr-matches atts">
				<td>
					<label for="matches-matchtype-id" onclick="toggle_select_row( this, 'matches' )"><?php _e( 'Select one or more match types', 'football-pool' ); ?></label>
				</td>
				<td>
					<select id="matches-matchtype-id" style="width:320px; height:100px; display:none;" multiple="multiple">
					<?php
					$match_types = Football_Pool_Matches::get_match_types();
					foreach( $match_types as $match_type ) {
						printf( '<option value="%d">%s</option>', $match_type->id, $match_type->name );
					}
					?>
					</select>
				</td>
			</tr>
			<tr class="tr-matches atts">
				<td>
					<label for="matches-group-id" onclick="toggle_select_row( this, 'matches' )"><?php _e( 'Select a group', 'football-pool' ); ?></label>
				</td>
				<td>
					<select id="matches-group-id" style="width:320px; display:none;">
					<option value=""></option>
					<?php group_options(); ?>
					</select>
				</td>
			</tr>
			<!-- fp-next-matches -->
			<tr class="tr-next-matches atts">
				<td>
					<label for="next-matches-matchtype-id"><?php _e( 'Select one or more match types', 'football-pool' ); ?></label>
				</td>
				<td>
					<select id="next-matches-matchtype-id" style="height:100px;" multiple="multiple">
					<?php
					$match_types = Football_Pool_Matches::get_match_types();
					foreach( $match_types as $match_type ) {
						printf( '<option value="%d">%s</option>', $match_type->id, $match_type->name );
					}
					?>
					</select>
				</td>
			</tr>
			<tr class="tr-next-matches atts">
				<td>
					<label for="next-matches-group-id"><?php _e( 'Select a group', 'football-pool' ); ?></label>
				</td>
				<td>
					<select id="next-matches-group-id">
					<option value=""></option>
					<?php group_options(); ?>
					</select>
				</td>
			</tr>
			<tr class="tr-next-matches atts">
				<td>
					<label><a href="//php.net/manual/en/function.date.php" title="<?php _e( 'information about PHP\'s date format', 'football-pool' ); ?>" target="_blank"><?php _e( 'Date', 'football-pool' ); ?></a></label>
				</td>
				<td>
					<label><input type="radio" id="next-matches-now" name="next-matches-date" value="now" checked="checked" /> <?php _e( 'now', 'football-pool' ); ?></label><br />
					<label><input type="radio" id="next-matches-postdate" name="next-matches-date" value="postdate" /> <?php _e( 'postdate', 'football-pool' ); ?></label><br />
					<label><input type="radio" id="next-matches-custom" name="next-matches-date" value="custom" onclick="jQuery( '#next-matches-custom-value' ).focus();" /> <?php _e( 'custom date', 'football-pool' ); ?>:</label> <input type="text" id="next-matches-custom-value" placeholder="Y-m-d H:i" onclick="jQuery( '#next-matches-custom' ).prop( 'checked', true );" /><br />
				</td>
			</tr>
			<tr class="tr-next-matches atts">
				<td>
					<label for="next-matches-num"><?php _e( 'Number of matches', 'football-pool' ); ?></label>
				</td>
				<td>
					<input type="text" id="next-matches-num" placeholder="5" />
				</td>
			</tr>
			<!-- fp-league-info -->
			<tr class="tr-league-info atts">
				<td>
					<label for="league-info-league-id"><?php _e( 'Select a league', 'football-pool' ); ?></label>
				</td>
				<td>
					<select id="league-info-league-id">
						<?php
						$leagues = $pool->get_leagues( true );
						foreach ( $leagues as $league ) {
							printf( '<option value="%d">%s</option>'
									, $league['league_id'], $league['league_name'] 
							);
						}
						?>
					</select>
				</td>
			</tr>
			<tr class="tr-league-info atts">
				<td>
					<label><?php _e( 'Show this info', 'football-pool' ); ?></label>
				</td>
				<td>
					<label><input type="radio" id="league-info-name" name="league-info-info" value="name" checked="checked" /> <?php _e( 'name', 'football-pool' ); ?></label><br />
					<label><input type="radio" id="league-info-points" name="league-info-info" value="points" /> points</label><br />
					<label><input type="radio" id="league-info-avgpoints" name="league-info-info" value="avgpoints" /> <?php _e( 'average points', 'football-pool' ); ?></label><br />
					<label><input type="radio" id="league-info-wavgpoints" name="league-info-info" value="wavgpoints" /> <?php _e( 'weighted average points', 'football-pool' ); ?></label><br />
					<label><input type="radio" id="league-info-playernames" name="league-info-info" value="playernames" /> <?php _e( 'player names', 'football-pool' ); ?></label><br />
					<label><input type="radio" id="league-info-numplayers" name="league-info-info" value="numplayers" /> <?php _e( 'number of players', 'football-pool' ); ?></label><br />
				</td>
			</tr>
			<tr class="tr-league-info atts">
				<td>
					<label for="league-info-ranking-id"><?php _e( 'Select a ranking', 'football-pool' ); ?></label>
				</td>
				<td>
					<select id="league-info-ranking-id">
						<optgroup label="<?php _e( 'default', 'football-pool' ); ?>">
							<option value="0" selected="selected"><?php _e( 'all scores', 'football-pool' ); ?></option>
						</optgroup>
						<optgroup label="<?php _e( 'or choose a user defined ranking', 'football-pool' ); ?>">
							<?php ranking_options(); ?>
						</optgroup>
					</select>
				</td>
			</tr>
			<tr class="tr-league-info atts">
				<td>
					<label for="league-info-format"><a href="//php.net/manual/en/function.sprintf.php" target="_blank"><?php _e( 'Format', 'football-pool' ); ?></a></label>
				</td>
				<td>
					<input type="text" id="league-info-format" />
				</td>
			</tr>
			</table>
		</div>
		
		<!-- panel -->
		<div id="links_panel" class="panel"><br />
			<table class="dialog-table">
			<tr class="shortcode-select">
				<td><label for="s-link"><?php _e( 'Select a shortcode', 'football-pool' ); ?></label></td>
				<td>
					<select id="s-link" class="shortcode" onchange="toggle_atts( this.id, { 'fp-link': 'tr-slug', 'fp-register': 'tr-title' } )">
						<option value="fp-link"><?php _e( 'Link to page', 'football-pool' ); ?></option>
						<option value="fp-register"><?php _e( 'Link to registration', 'football-pool' ); ?></option>
					</select>
				</td>
			</tr>
			<tr class="tr-slug">
				<td><label for="slug"><?php _e( 'Select a page', 'football-pool' ); ?></label></td>
				<td>
					<select id="slug">
					<?php
					$pages = Football_Pool::get_pages();
					foreach ( $pages as $page ) {
						printf( '<option value="%s">%s</option>', $page['slug'], __( $page['title'], 'football-pool' ) );
					}
					?>
					</select>
				</td>
			</tr>
			<tr class="tr-title atts">
				<td>
					<p><label for="link-title"><?php _e( 'Link title', 'football-pool' ); ?></label></p>
					<p><label for="link-window"><?php _e( 'New window?', 'football-pool' ); ?></label></p>
				</td>
				<td>
					<p><input id="link-title" type="text" /></p>
					<p><input id="link-window" type="checkbox" /></p>
				</td>
			</tr>
			</table>
		</div>
		
		<!-- panel -->
		<div id="other_panel" class="panel"><br />
			<table class="dialog-table">
			<tr class="shortcode-select">
				<td><label for="s-other"><?php _e( 'Select a shortcode', 'football-pool' ); ?></label></td>
				<td>
					<select id="s-other" class="shortcode" onchange="toggle_atts( this.id, { 'fp-countdown': 'tr-count' } )">
						<option value="fp-countdown"><?php _e( 'Countdown', 'football-pool' ); ?></option>
						<option value="fp-jokermultiplier"><?php _e( 'Value for' ); ?> <?php _e( 'Joker multiplier', 'football-pool' ); ?></option>
						<option value="fp-fullpoints"><?php _e( 'Value for' ); ?> <?php _e( 'Full points', 'football-pool' ); ?></option>
						<option value="fp-totopoints"><?php _e( 'Value for' ); ?> <?php _e( 'Toto points', 'football-pool' ); ?></option>
						<option value="fp-goalpoints"><?php _e( 'Value for' ); ?> <?php _e( 'Goal bonus', 'football-pool' ); ?></option>
					</select>
				</td>
			</tr>
			<tr class="tr-count">
				<td>
					<label for=""><?php _e( 'Countdown to', 'football-pool' ); ?></label>
				</td>
				<td><label>
					<input type="radio" id="count-to-match" name="count_to" value="match" checked="checked" onclick="FootballPoolAdmin.toggle_linked_options( '#tr-count-match', '#tr-count-date' )" /> <?php _e( 'Match', 'football-pool' ); ?></label><br />
					<label><input type="radio" id="count-to-date" name="count_to" value="date" onclick="FootballPoolAdmin.toggle_linked_options( '#tr-count-date', '#tr-count-match' )" /> <?php _e( 'Date', 'football-pool' ); ?></label><br />
				</td>
			</tr>
			<tr id="tr-count-date" class="tr-count atts">
				<td>
					<label for="count-date"><a href="//php.net/manual/en/function.date.php" title="<?php _e( 'information about PHP\'s date format', 'football-pool' ); ?>" target="_blank"><?php _e( 'Date', 'football-pool' ); ?></a></label>
				</td>
				<td>
					<input type="text" id="count-date" placeholder="Y-m-d H:i" />
				</td>
			</tr>
			<tr id="tr-count-match" class="tr-count">
				<td>
					<label for="count-match"><?php _e( 'Match', 'football-pool' ); ?></label>
				</td>
				<td>
					<select id="count-match" style="width:320px">
						<optgroup label="<?php _e( 'default', 'football-pool' ); ?>">
							<option value="0" selected="selected"><?php _e( 'first match', 'football-pool' ); ?></option>
						</optgroup>
						<optgroup label="<?php _e( 'other', 'football-pool' ); ?>">
							<option value="next"><?php _e( 'next match', 'football-pool' ); ?></option>
						</optgroup>
						<optgroup label="<?php _e( 'or choose a match', 'football-pool' ); ?>">
							<?php match_options(); ?>
						</optgroup>
					</select>
				</td>
			</tr>
			<tr class="tr-count">
				<td>
					<label for="text-1"><a target="_blank" href="<?php echo $admin_url; ?>admin.php?page=footballpool-help#shortcodes" title="<?php _e( 'More information about this on the Help page.', 'football-pool' ); ?>"><?php _e( 'Texts for counter', 'football-pool' ); ?></a></label>
				</td>
				<td>
					<p>
						<label><input type="checkbox" id="count-no-texts" value="1" onchange="toggle_count_texts( this.id )" /> <?php _e( 'no texts', 'football-pool' ); ?></label>
					</p>
					<p>
						<input type="text" id="text-1" placeholder="<?php _e( 'before - time not passed', 'football-pool' ); ?>" title="<?php _e( "Leave empty for the default texts. Don't forget spaces between a text and the timer.", 'football-pool' ); ?>" />
						<input type="text" id="text-2" placeholder="<?php _e( 'after - time not passed', 'football-pool' ); ?>" title="<?php _e( "Leave empty for the default texts. Don't forget spaces between a text and the timer.", 'football-pool' ); ?>" />
					<br />
						<input type="text" id="text-3" placeholder="<?php _e( 'before - time passed', 'football-pool' ); ?>" title="<?php _e( "Leave empty for the default texts. Don't forget spaces between a text and the timer.", 'football-pool' ); ?>" />
						<input type="text" id="text-4" placeholder="<?php _e( 'after - time passed', 'football-pool' ); ?>" title="<?php _e( "Leave empty for the default texts. Don't forget spaces between a text and the timer.", 'football-pool' ); ?>" />
					</p>
				</td>
			</tr>
			<tr class="tr-count">
				<td>
					<label for="count-inline"><?php _e( 'Display inline', 'football-pool' ); ?></label>
				</td>
				<td>
					<input id="count-inline" type="checkbox" />
				</td>
			</tr>
			<tr class="tr-count">
				<td>
					<label for="count-format"><?php _e( 'Time format', 'football-pool' ); ?></label>
				</td>
				<td>
					<select id="count-format">
						<option value="2"><?php _e( 'days, hours, minutes, seconds', 'football-pool' ); ?></option>
						<option value="3"><?php _e( 'hours, minutes, seconds', 'football-pool' ); ?></option>
						<option value="1"><?php _e( 'only seconds', 'football-pool' ); ?></option>
					</select>
				</td>
			</tr>
			</table>
		</div>
		
	</div>
	
	<div class="mceActionPanel">
		<div style="float: left">
			<input type="button" id="cancel" name="cancel" value="<?php _e( 'Cancel', 'football-pool' ); ?>" onclick="tinyMCEPopup.close();" />
		</div>
		<div style="float: right">
			<input type="submit" id="insert" name="insert" value="<?php _e( 'Insert', 'football-pool' ); ?>" onclick="FootballPoolAdmin.tinymce_insert_shortcode();" />
		</div>
	</div>

</form>
</body>
</html>